package com.hs.api.service.Impl;

import com.hs.api.common.Constants;
import com.hs.api.common.enums.SysConfigKeyType;
import com.hs.api.common.exceptions.DBConfigurationError;
import com.hs.api.common.utils.AESUtil;
import com.hs.api.common.utils.DateUtils;
import com.hs.api.common.utils.SessionUtil;
import com.hs.api.common.utils.TokenUtil;
import com.hs.api.mapper.DicOrgMapper;
import com.hs.api.mapper.SysRoleMapper;
import com.hs.api.mapper.SysUserMapper;
import com.hs.api.mapper.SysVersionMapper;
import com.hs.api.model.*;
import com.hs.api.model.respmodel.LoginInfo;
import com.hs.api.service.LoginService;
import com.hs.api.service.SysConfigService;
import com.hs.api.service.UserService;
import com.hs.common.utils.RedisUtil;
import com.hs.common.utils.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;


@Service
public class LoginServiceImpl implements LoginService {

    @Autowired
    private UserService userServiceImpl;

    @Autowired
    private SysConfigService sysConfigService;

    @Resource
    private SysUserMapper sysUserMapper;
    @Resource
    private DicOrgMapper dicOrgMapper;
    @Resource
    private SysRoleMapper sysRoleMapper;
    @Resource
    private SysVersionMapper sysVersionMapper;

    public final static long FOREVER_DURATION = -1;

    @Override
    public LoginInfo login(String logName, String password, String macAddrss, String ip) {

        LoginInfo loginInfo = new LoginInfo();

        SysUser user = userServiceImpl.findByLoginName(logName);
        //判断是否被锁定
        if(user != null && user.isLockFlag())
            throw new DBConfigurationError("该账号已被锁定请联系管理员解锁！");
        //判断是否很久没有更改密码
        if(sysConfigService.getStateByKey(SysConfigKeyType.PWD_TIME_LIMIT.getCode())
                && DateUtils.getDaysBetween(user.getLastUpdatePwdTime(), new Date()) > 90)
            throw new DBConfigurationError("您已经超过90天没有更换密码，请修改密码后再登录！");
        if (user != null && user.getPassword().equals(AESUtil.encrypt(password)) && (user.getMacAddress() == null || user.getMacAddress().equals(macAddrss))) {
            String token = TokenUtil.getToken(AESUtil.encrypt(password), user);
            setUserTokenList(token, user);
            //修改ip以及最后登录时间、MAC地址
            user.setLastIp(ip);
            user.setLastTime(new Date());
            user.setMacAddress(macAddrss);
            sysUserMapper.updateByPrimaryKey(user);

            user.setPassword(null);
            loginInfo.setToken(token);
            loginInfo.setUser(user);

            //查询用户角色
            SysRole role = sysRoleMapper.selectByUser(user.getId());
            loginInfo.setRole(role);
            //查询用户机构
            List<DicOrg> orgList = dicOrgMapper.selectByUser(user.getId());
            if (orgList != null && orgList.size() > 0) {
                List<SerDepartment> deptList = dicOrgMapper.selectDeptByUser(user.getId(), orgList.get(0).getId());
                loginInfo.setDept(deptList != null && deptList.size() > 0 ? deptList.get(0) : null);
            }
            loginInfo.setOrg(orgList);

            HttpSession session = SessionUtil.getSession();
            session.setAttribute("userId", user.getId());
            session.setAttribute("token", token);

            //查询病案信息
            SysVersion version = sysVersionMapper.selectLastDate();
            //add 检查发布功能  by mengchao 2021-05-24 begin
            if(sysConfigService.getStateByKey(SysConfigKeyType.BAH_CXRQ_CONTROL.getCode()) && !user.getIsAdmin())
            {
                version.setBahDate(version.getBahDate()-1);
            }
            // add end
            loginInfo.setVersion(version);
            RedisUtil.del(user.getUserCode());
        } else {
            if(user != null) checkErrorCount(user);
            throw new DBConfigurationError("登录失败请检查用户名和密码！");
        }
        return loginInfo;
    }

    //多重并发控制
    private  void setUserTokenList(String token, SysUser user) {
        String value = sysConfigService.getValueByKey(SysConfigKeyType.SIMULTAN_ONLINE_NUM.getCode());
        boolean sysConfigStateByKey = sysConfigService.getStateByKey(SysConfigKeyType.SIMULTAN_ONLINE_NUM.getCode());
        if(!sysConfigStateByKey) return;
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String path = request.getServletContext().getContextPath();
        List<String> tokenList = RedisUtil.get(user.getId() + path) == null?
                new ArrayList<>():(List<String>)RedisUtil.get(user.getId() + path);
        if(!tokenList.contains(token) && tokenList.size()<Integer.parseInt(value)) {
            tokenList.add(token);
            RedisUtil.set(user.getId() + path, tokenList, FOREVER_DURATION);
        }else if(!tokenList.contains(token)) {
            tokenList.remove(0);
            tokenList.add(token);
            RedisUtil.set(user.getId() + path, tokenList, FOREVER_DURATION);
        }
    }

    private void checkErrorCount(SysUser user) {
        String userCode = user.getUserCode();
        int configTimes = Integer.parseInt(sysConfigService.getValueByKey(SysConfigKeyType.LOGIN_ERROR_TIMES.getCode())) - 1;
        if(!sysConfigService.getStateByKey(SysConfigKeyType.LOGIN_ERROR_TIMES.getCode()))
            return;
        if(RedisUtil.hasKey(userCode)
                && Integer.parseInt(RedisUtil.get(userCode).toString()) >= configTimes) {
            user.setLockFlag(true);
            sysUserMapper.updateByPrimaryKey(user);
            return;
        }
        if(!RedisUtil.hasKey(userCode)) {
            RedisUtil.set(userCode, 1, 5 * 60);
        }else {
            RedisUtil.incr(userCode, 1);
        }
    }

    @Override
    public LoginInfo logout() {
        LoginInfo loginInfo = new LoginInfo();
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String path = request.getServletContext().getContextPath();
        String token = request.getHeader(Constants.TOKEN_KEY);
        SysUser user = (SysUser) RedisUtil.get(token);
        loginInfo.setUser(user);
        loginInfo.setToken(token);
        if(!token.equals("") && token!=null) {
            RedisUtil.del(token);
        };

        if(user!=null && RedisUtil.get(user.getId() + path)!=null) {
            List<String> tokenList = (List<String>)RedisUtil.get(user.getId() + path);
            if(tokenList.contains(token))
                tokenList.remove(token);
        }


        return loginInfo;
    }

    public boolean updatePass(String logName, String oldPassword, String newPassword) {
        int result = 0;
        SysUser user = userServiceImpl.findByLoginName(logName);
        if (user != null && user.getPassword().equals(AESUtil.encrypt(oldPassword))) {
            user.setPassword(AESUtil.encrypt(newPassword));
            user.setLastUpdatePwdTime(new Date());
            result = sysUserMapper.updateByPrimaryKey(user);
        } else {
            throw new DBConfigurationError("旧密码错误！");
        }
        return result > 0;
    }


    public String refreshToken(String oldToken) {
        return TokenUtil.refresh(oldToken);
    }
}
